from collections import deque


class AVLTreeNode:
    def __init__(self, val=None):
        self.val = val
        self.l_child = None
        self.r_child = None
        self.parent = None
        self.bf = 0


class AVLTree:
    def __init__(self, li=None):
        self.root = None
        if li:
            for val in li:
                self.insert_two(val)

    def tree_left(self, p, c):
        s2 = c.l_child
        p.r_child = s2
        if s2:
            s2.parent = p

        c.l_child = p
        p.parent = c

        c.bf = 0
        p.bf = 0

        return c

    def tree_right(self, p, c):
        s2 = c.r_child
        p.l_child = s2
        if s2:
            s2.parent = p

        c.r_child = p
        p.parent = c

        p.bf = 0
        c.bf = 0

        return c

    def tree_right_left(self, p, c):
        g = c.l_child
        s2 = g.r_child
        c.l_child = s2
        if s2:
            s2.parent = c

        g.r_child = c
        c.parent = g

        s3 = g.l_child
        p.r_child = s3
        if s3:
            s3.parent = p

        g.l_child = p
        p.parent = g

        if g.bf > 0:
            c.bf = 0
            p.bf = -1
        elif g.bf < 0:
            p.bf = 0
            c.bf = 1
        else:
            p.bf = 0
            c.bf = 0

        return g

    def tree_left_right(self, p, c):
        g = c.r_child

        s2 = g.l_child
        c.r_child = s2
        if s2:
            s2.parent = c

        g.l_child = c
        c.parent = g

        s3 = g.r_child
        p.l_child = s3
        if s3:
            s3.parent = p

        g.r_child = p
        p.parent = g

        if g.bf > 0:
            p.bf = 0
            c.bf = -1
        elif g.bf < 0:
            p.bf = 1
            c.bf = 0
        else:
            p.bf = 0
            c.bf = 0

        return g

    def insert(self, val_):
        def insert_(node, val):
            if node:
                if val < node.val:
                    if node.l_child:
                        insert_(node.l_child, val)
                    else:
                        node.l_child = AVLTreeNode(val)
                        node.l_child.parent = node
                else:
                    if node.r_child:
                        insert_(node.r_child, val)
                    else:
                        node.r_child = AVLTreeNode(val)
                        node.r_child.parent = node
            else:
                node = AVLTreeNode(val)

            return node

        self.root = insert_(self.root, val_)

    def insert_two(self, val):
        # 插入数据
        p = self.root
        while p:
            if val < p.val:
                if p.l_child:
                    p = p.l_child
                else:
                    p.l_child = AVLTreeNode(val)
                    p.l_child.parent = p
                    new_node = p.l_child
                    break
            else:
                if p.r_child:
                    p = p.r_child
                else:
                    p.r_child = AVLTreeNode(val)
                    p.r_child.parent = p
                    new_node = p.r_child
                    break
        else:
            self.root = AVLTreeNode(val)
            return

        # 更新bf
        while new_node.parent:
            if new_node.parent.l_child == new_node:
                if new_node.parent.bf == -1:
                    parent_node_two = new_node.parent.parent
                    x = new_node.parent
                    if new_node.bf > 0:
                        n = self.tree_left_right(new_node.parent, new_node)
                    else:
                        n = self.tree_right(new_node.parent, new_node)
                elif new_node.parent.bf == 0:
                    new_node.parent.bf = -1
                    new_node = new_node.parent
                    continue
                else:
                    new_node.parent.bf = 0
                    break
            else:
                if new_node.parent.bf == 1:
                    parent_node_two = new_node.parent.parent
                    x = new_node.parent
                    if new_node.bf > 0:
                        n = self.tree_left(new_node.parent, new_node)
                    else:
                        n = self.tree_right_left(new_node.parent, new_node)
                elif new_node.parent.bf == 0:
                    new_node.parent.bf = 1
                    new_node = new_node.parent
                    continue
                else:
                    new_node.parent.bf = 0
                    break

            n.parent = parent_node_two
            if parent_node_two:
                if parent_node_two.l_child == x:
                    parent_node_two.l_child = n
                else:
                    parent_node_two.r_child = n
            else:
                self.root = n
            break

    def select(self, val):
        p = self.root
        while p:
            if p.val == val:
                return p
            elif p.val > val and p.l_child:
                p = p.l_child
            elif p.val < val and p.r_child:
                p = p.r_child
            else:
                return None

        return None

    def __remove_node1(self, node):
        if not node.parent:
            self.root = None
            return
        if node.parent.l_child == node:
            node.parent.l_child = None
        else:
            node.parent.r_child = None

    def __remove_node2(self, node):
        if node.l_child:
            if not node.parent:
                self.root = node.l_child
            else:
                if node.parent.l_child == node:
                    node.parent.l_child = node.l_child
                else:
                    node.parent.r_child = node.l_child
        else:
            if not node.parent:
                self.root = node.r_child
            else:
                if node.parent.l_child == node:
                    node.parent.l_child = node.r_child
                else:
                    node.parent.r_child = node.r_child

    def delete(self, val):
        if not self.root:
            return False

        node = self.select(val)
        if not node:
            return False

        # 删除后还需要更新bf
        if not node.l_child and not node.r_child:
            self.__remove_node1(node)
            return True
        elif not node.l_child or not node.r_child:
            self.__remove_node2(node)
            return True
        else:
            min_node = node.r_child
            while min_node.l_child:
                min_node = min_node.l_child
            node.val = min_node.val

            if min_node.r_child:
                self.__remove_node2(min_node)
            else:
                self.__remove_node1(min_node)

            return True

    def show(self, s1=" "):
        print("二叉树形式输出：")
        datas = [[]]
        data = []
        i = 0
        if self.root:
            data = [self.root]
        while True:
            data1 = []
            for p in data:
                if p:
                    datas[i].append(p.val)
                    data1 += [p.l_child, p.r_child]
                else:
                    datas[i].append(None)
                    data1 += [None, None]

            for x in data1:
                if x:
                    break
            else:
                break

            data = data1
            i += 1
            datas.append([])

        n = len(datas)
        all_len = 2 * (2 ** n)
        for i in range(n):
            str1 = ""
            for item in datas[i]:
                if item == None:
                    item = s1
                str1 += str(item).center(all_len // (2 ** i))
            print(str1.center(all_len), end="\n\n")

    def show_1(self):
        def show_1_(node):
            if node:
                data.append(node.val)
                show_1_(node.l_child)
                show_1_(node.r_child)

        data = []
        show_1_(self.root)
        print("前序遍历结果：", data)

    def show_2(self):
        def show_2_(node):
            if node:
                show_2_(node.l_child)
                data.append(node.val)
                show_2_(node.r_child)

        data = []
        show_2_(self.root)
        print("中序遍历结果：", data)

    def show_3(self):
        def show_3_(node):
            if node:
                show_3_(node.l_child)
                show_3_(node.r_child)
                data.append(node.val)

        data = []
        show_3_(self.root)
        print("后序遍历结果：", data)

    def show_4(self):
        data = []
        q = deque()
        p = self.root

        if p:
            q.append(p)

        while len(q) > 0:
            p1 = q.popleft()
            data.append([p1.val, p1.bf])
            if p1.l_child:
                q.append(p1.l_child)
            if p1.r_child:
                q.append(p1.r_child)

        print("层次遍历结果：", data)

import random
li = list(range(1, 20))
random.shuffle(li)
tt = AVLTree(li)
tt.show("X")
tt.show_4()

print("li: ", li)

